﻿using Microsoft.Extensions.Logging;
using SushiBarContracts.BindingModels;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.SearchModels;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;

namespace SushiBarBusinessLogic.BusinessLogics
{
    public class IngredientLogic : IIngredientLogic
    {
        private readonly ILogger _logger;
        private readonly IIngredientStorage _componentStorage;
        public IngredientLogic(ILogger<IngredientLogic> logger, IIngredientStorage componentStorage)
        {
            _logger = logger;
            _componentStorage = componentStorage;
        }
        public List<IngredientViewModel>? ReadList(IngredientSearchModel? model)
        {
            _logger.LogInformation("ReadList. IngredientName: {IngredientName}. Id: {Id}",
                model?.IngredientName, model?.Id);
            var list = model == null ? _componentStorage.GetFullList() :
                _componentStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }
        public IngredientViewModel? ReadElement(IngredientSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. IngredientName: {IngredientName}. Id: {Id}",
                model.IngredientName, model.Id);
            var element = _componentStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id: {Id}", element.Id);
            return element;
        }
        public bool Create(IngredientBindingModel model)
        {
            CheckModel(model);
            if (_componentStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(IngredientBindingModel model)
        {
            CheckModel(model);
            if (_componentStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(IngredientBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id: {Id}", model.Id);
            if (_componentStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(IngredientBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.IngredientName))
            {
                throw new ArgumentNullException("Нет названия ингредиента", nameof(model.IngredientName));
            }
            if (model.Cost <= 0)
            {
                throw new ArgumentNullException("Цена ингредиента должна быть больше 0",
                    nameof(model.Cost));
            }
            _logger.LogInformation("Ingredient. IngredientName: {IngredientName}. Cost: {Cost}. Id: {Id}",
                model.IngredientName, model.Cost, model.Id);
            var element = _componentStorage.GetElement(new IngredientSearchModel
            {
                IngredientName = model.IngredientName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Ингридиент с таким названием уже есть");
            }
        }
    }
}